/*
 * MODULE: dfopen.c
*/

#include <stdio.h>
#include <string.h>
#include "pstr.h"

/*
 * Oeffne Datei mit Suchpfadangabe:
 *	Diese Funktion erhaelt als erstes Argument eine Liste
 *	von Directories (Namen durch Doppelpunkt getrennt,
 *	wie z.B. in PATH).  In diesen Directories wird nach
 *	einer Datei gesucht, deren Namen das zweite Argument
 *	angibt.  Die Suche endet, wenn sich eine solche Datei
 *	mit dem durch das dritte Argument angegebenen Modus
 *	oeffnen laesst.  In diesem Fall ist das Ergebnis der
 *	FILE-Pointer, mit dem sich auf die Datei zugreifen
 *	laesst.  Wird keine Datei gefunden, ist das Ergebnis
 *	ein (FILE *)0-Zeiger.
*/

FILE *dfopen(
	const char *dirlist,	/* => Verzeichnisliste */
	const char *name,	/* => Dateiname */
	const char *mode	/* => Oeffnungsmodus */
) {			/* <= FILE-Pointer oder (FILE *)0 */
	FILE *fp = (FILE *)0;
	static const char SEP[] = ":";
	char *d = (char *)pstr_1(dirlist);
	/*CASTAWAYCONST*/
	char *dir = strtok((char *)d, SEP);
	while (dir != (char *)0) {
		const char *pathname = pstr_3(dir, "/", name);
		fp = fopen(pathname, mode);
		pstr_delete(pathname);
		if (fp != (FILE *)0)
			break;
		dir = strtok((char *)0, ":");
	}
	pstr_delete(d);

	return fp;
}

#ifdef TEST

#include <ctype.h>

int main(int argc, char *argv[])
{
	FILE *fp;
	int c;
	if (argc == 3) {
		if ((fp = dfopen(argv[1], argv[2], "r")) != (FILE *)0) {
			while ((c = getc(fp)) != EOF) {
				printf(c == '\t' || c == '\n' || isprint(c)
					? "%c" : "\\%03o", (unsigned char)c);
			}
		}
	}
	return 0;
}

#endif
